<?php
namespace Hal\Violation\Class_;

use Hal\Metric\ClassMetric;
use Hal\Metric\Metric;
use Hal\Violation\Violation;

/**
 * Class TooDependent
 *
 * @package Hal\Violation\Class_
 */
class TooDependent implements Violation
{

    /**
     * @inheritdoc
     */
    public function getName()
    {
        return 'Too dependent';
    }

    /**
     * @inheritdoc
     */
    public function apply(Metric $metric)
    {
        if (!$metric instanceof ClassMetric) {
            return;
        }

        $this->metric = $metric;

        if ($metric->get('efferentCoupling') >= 20) {
            $metric->get('violations')->add($this);
            return;
        }
    }

    /**
     * @inheritdoc
     */
    public function getLevel()
    {
        return Violation::INFO;
    }

    /**
     * @inheritdoc
     */
    public function getDescription()
    {
        $efferentCoupling = $this->metric->get('efferentCoupling');

        return <<<EOT
This class looks use really high number of components.

* Efferent coupling is {$efferentCoupling}, so this class uses {$efferentCoupling} different external components.

Maybe you should check why this class has lot of dependencies.
EOT;
    }
}
